'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var isTypedarray      = isTypedArray;
isTypedArray.strict = isStrictTypedArray;
isTypedArray.loose  = isLooseTypedArray;

var toString = Object.prototype.toString;
var names = {
    '[object Int8Array]': true
  , '[object Int16Array]': true
  , '[object Int32Array]': true
  , '[object Uint8Array]': true
  , '[object Uint8ClampedArray]': true
  , '[object Uint16Array]': true
  , '[object Uint32Array]': true
  , '[object Float32Array]': true
  , '[object Float64Array]': true
};

function isTypedArray(arr) {
  return (
       isStrictTypedArray(arr)
    || isLooseTypedArray(arr)
  )
}

function isStrictTypedArray(arr) {
  return (
       arr instanceof Int8Array
    || arr instanceof Int16Array
    || arr instanceof Int32Array
    || arr instanceof Uint8Array
    || arr instanceof Uint8ClampedArray
    || arr instanceof Uint16Array
    || arr instanceof Uint32Array
    || arr instanceof Float32Array
    || arr instanceof Float64Array
  )
}

function isLooseTypedArray(arr) {
  return names[toString.call(arr)]
}

/* globals unknown */
/**
 * @typedef {Int8Array | Int16Array | Int32Array | Uint8Array | Uint8ClampedArray | Uint16Array | Uint32Array | Float32Array | Float64Array} TypedArray
 */

/**
 * Returns value at specified index. Supports relative indexing from the end when passed a negative index. Uses native implementation if available.
 *
 * @param {string|Array<unknown>|TypedArray} item
 * @param {number}                           [index]
 */

function ponyfill(item, index) {
  var _index;

  if (typeof item !== 'string' && !Array.isArray(item) && !isTypedarray.loose(item)) {
    throw new TypeError('Expected a string or array-like item.');
  }

  index = Math.trunc((_index = index) !== null && _index !== void 0 ? _index : 0) || 0;

  if (index < 0) {
    index += item.length;
  }

  if (index < 0 || index >= item.length) {
    return undefined;
  }

  return item[index];
}
/**
 * Returns value at specified index. Supports relative indexing from the end when passed a negative index. Uses native implementation if available.
 *
 * @param {string|Array<unknown>|TypedArray} item    Item to search.
 * @param {number}                           [index] Relative index.
 */


function preferNative(item, index) {
  var _item$at, _item$at2;

  return (_item$at = item === null || item === void 0 ? void 0 : (_item$at2 = item.at) === null || _item$at2 === void 0 ? void 0 : _item$at2.call(item, index !== null && index !== void 0 ? index : 0)) !== null && _item$at !== void 0 ? _item$at : ponyfill(item, index);
}

exports['default'] = ponyfill;
exports.preferNative = preferNative;
//# sourceMappingURL=index.js.map
